dnl @synopsis CHECK_LIBMHASH([ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
dnl
dnl This macro searches for an installed libmhash library. If nothing
dnl was specified when calling configure, it searches first in /usr/local
dnl and then in /usr. If the --with-libmhash=DIR is specified, it will try
dnl to find it in DIR/include/libmhash.h and DIR/lib/libmhash.a. If --without-libmhash
dnl is specified, the library is not searched at all.
dnl
dnl It defines the symbol HAVE_LIBMHASH if the library is found. You should
dnl use autoheader to include a definition for this symbol in a config.h
dnl file.
dnl
dnl Sources files should then use something like
dnl
dnl #ifdef HAVE_LIBMHASH
dnl #include <libmhash.h>
dnl #endif /* HAVE_LIBMHASH */
dnl
dnl @version 1.0
dnl based on CHECK_ZLIB by Loic Dachary <loic@senga.org>
dnl

AC_DEFUN(CHECK_LIBMHASH,
#
# Handle user hints
#
         [AC_MSG_CHECKING(if libmhash is wanted)
         AC_ARG_WITH(libmhash,
                     [  --with-libmhash=DIR  root directory path of libmhash installation [defaults to
                          /usr/local or /usr if not found in /usr/local]
  --without-libmhash   to disable libmhash usage completely],
                     [if test "$withval" != no ; then
                          LIBMHASH_HOME="$withval"
                          AC_MSG_RESULT([yes: libraries ${LIBMHASH_HOME}/lib  includes ${LIBMHASH_HOME}/include])
                      else
                          AC_MSG_RESULT(no)
                      fi],
                     [LIBMHASH_HOME=/usr/local
                      if test ! -f "${LIBMHASH_HOME}/include/libmhash.h"
                      then
                         LIBMHASH_HOME=/usr
                      fi
                      AC_MSG_RESULT([yes: libraries ${LIBMHASH_HOME}/lib  includes ${LIBMHASH_HOME}/include])
                     ])

#
# Locate libmhash, if wanted
#
         if test -n "${LIBMHASH_HOME}"
         then
             LDFLAGS="$LDFLAGS -L${LIBMHASH_HOME}/lib"
             CPPFLAGS="$CPPFLAGS -I${LIBMHASH_HOME}/include"
             AC_CHECK_LIB(mhash, mhash_init, $1, $2)
         fi
    ])
