#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h> 
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <mcrypt.h>

#include "crypto-0.7.1.h"
#include "pasmanedit.h"
#include "wmpasman.h"
#include "crypto.h"
#include "die.h"
#include "globals.h"
#include "secure.h"
#include "password.h"
#include "filestuff.h"

void LoadOldPasswordFile(PWList list, const char *filename){
    int fp;
    char *iv;
    int len, i;
    void *cry;
    char l1[10], l2[10];
    unsigned char buf[21];
    Password p;
    
    if((iv=getIVBuf(&i))==NULL) die("Couldn't get a Crypter IV");
    
    fp=open(filename, O_RDONLY);
    if(fp<0){
        free(iv);
        die("Couldn't open %s", filename);
    }

    if(i!=0){
        len=read(fp, iv, i);
        if(len!=0 && len<i){
            free(iv);
            close(fp);
            die("Error(1) reading %s", filename);
        } else if(len==0){
            free(iv);
            close(fp);
            return;
        }
    }

    cry=newCrypter(iv);
    free(iv);
    if(cry==NULL){
        close(fp);
        die("Couldn't create a v0.7.1 Crypter");
    }

    l1[9]=0; l2[9]=0;
    while((len=read(fp, buf, 20))!=0){
        if(len<20) die("Error(2) reading %s", filename);
        decryptString(cry, buf, 20);
        len=(buf[0]<<8)+buf[1];
        memcpy(l1, buf+2, 9);
        memcpy(l2, buf+11, 9);
        if(len>=MAX_PASSWORD_LEN) die("Password \"%s\" \"%s\" is too long", l1, l2);
        if(read(fp, password_buffer, len)<len) die("Error(3) reading %s", filename);
        decryptString(cry, password_buffer, len);
        password_buffer[len]='\0';
        if((p=password_new(l1, l2, password_buffer))==NULL) die("memory allocation");
        if(pwlist_push(list, p)) die("memory allocation");
    }
    freeCrypter(cry);
    close(fp);
}

int get_passphrase(void){
    char **algorithms;
    int numalgo, i, maxwidth, width;
    char *s;

    algorithms=mcrypt_list_algorithms(NULL, &numalgo);
    if(!algorithms) die("memory allocation");
    errno=0;
    if(numalgo<=0) die("libmcrypt reports no algorithms");
    maxwidth=0;
    for(i=0; i<numalgo; i++){
        width=strlen(algorithms[i]);
        if(maxwidth<width) maxwidth=width;
    }
    maxwidth+=4;

    width=0;
    printf("Available algorithms\n");
    printf("====================\n");
    for(i=0; i<numalgo; i++){
        printf("%-*s", maxwidth, algorithms[i]);
        width+=maxwidth;
        if(width+maxwidth>CONSOLE_WIDTH){
            printf("\n");
            width=0;
        }
    }
    if(width) printf("\n");
    printf("Choose Cypher: ");
    fflush(stdout);
    fgets(passphrase_buffer2, MAX_PHRASEHASH_LEN, stdin);
    s=passphrase_buffer2+strlen(passphrase_buffer2)-1;
    if(*s=='\n') *s=0;

    printf("Passphrase: ");
    fgets(password_buffer, MAX_PASSWORD_LEN, stdin);
    s=password_buffer+strlen(password_buffer)-1;
    if(*s=='\n') *s=0;

    if(crypto_set_passphrase(passphrase_buffer2, password_buffer)){
        debug(DEBUG_ERR, "Invalid cypher/passphrase");
        return -1;
    }
    if((s=expandKey(passphrase_buffer2, password_buffer))!=NULL)
        die("Couldn't set old-style passphrase: %s", s);
    return 0;
}

void usage(int exitcode) __attribute__ ((noreturn));

void usage(int exitcode){
    fprintf(stderr, "USAGE: %s [<options>]\n", PROGNAME);
    fprintf(stderr, "Options may be any of the following:\n");
    fprintf(stderr, "  -o, --old-pwfile <filename> File to load the v0.7.1 password from.\n");
    global_usage(25);
    exit(exitcode);
}

int main(int argc, char **argv){
    PWList list;
    char *filename;
    int i;
    
    if(init_globals(&argc, &argv)) usage(0);
    filename=PWFILE;
    i=1;
    if(argc>2 && (!strcmp(argv[1], "-o") || !strcmp(argv[1], "--old-pwfile"))){
       filename=argv[3];
       i=3;
       argc-=2;
    }
    if(argc!=1){
        debug(DEBUG_ERR, "Unknown parameter %s", argv[i]);
        usage(1);
    }

    while(get_passphrase());

    if((list=pwlist_new(0))==NULL) die("Couldn't make a PWList");
    LoadOldPasswordFile(list, filename);
    if(save_passwords(list)) die("Couldn't save passwords");
    return 0;
}
