#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <mcrypt.h>
#include <mhash.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "crypto-0.7.1.h"

char *crypto_key=NULL;
int crypto_key_len;
char *crypto_algo=NULL;
int crypto_algo_len;
static MCRYPT crypto_encrypt=MCRYPT_FAILED;
static MCRYPT crypto_decrypt=MCRYPT_FAILED;

char *expandKey(char *algo, char *pass){
    char *mode;
    void *iv;
    int i;

    if(crypto_encrypt!=MCRYPT_FAILED) forgetKey(1);

    if(algo==NULL){
	fprintf(stderr, "expandKey: You must select an algorithm!\n");
	return "expandKey: You must select an algorithm!";
    }
    if((i=mcrypt_module_self_test(algo, NULL))){
	fprintf(stderr, "expandKey: ");
	mcrypt_perror(i);
	return "expandKey: Mcrypt module self test failed";
    }
    mode=(mcrypt_module_is_block_algorithm(algo, NULL))?"cfb":"stream";

    crypto_algo_len=strlen(algo);
    crypto_key_len=mcrypt_module_get_algo_key_size(algo, NULL);
    if((crypto_algo=strdup(algo))==NULL) return strerror(errno);
    if((crypto_key=malloc(crypto_key_len))==NULL){
	i=errno;
	forgetKey(0);
	perror("expandKey");
	return strerror(i);
    }
    crypto_encrypt=mcrypt_module_open(algo, NULL, mode, NULL);
    if(crypto_encrypt==MCRYPT_FAILED){
	forgetKey(0);
	fprintf(stderr, "expandKey: mcrypt_module_open failed\n");
	return "expandKey: mcrypt_module_open failed";
    }
    crypto_decrypt=mcrypt_module_open(algo, NULL, mode, NULL);
    if(crypto_decrypt==MCRYPT_FAILED){
	forgetKey(0);
	fprintf(stderr, "expandKey: mcrypt_module_open failed\n");
	return "expandKey: mcrypt_module_open failed";
    }

    i=strlen(pass);
    if(mhash_keygen(KEYGEN_S2K_SIMPLE, MHASH_SHA1, i, crypto_key, crypto_key_len, NULL, 0, pass, i)<0){
	forgetKey(0);
	fprintf(stderr, "expandKey: Error hashing passphrase\n");
	return "expandKey: Error hashing passphrase";
    }
    fry(pass, i);

    if((iv=getIV(&i))==NULL){
	forgetKey(0);
	perror("expandKey");
	return strerror(errno);
    }

    if((i=mcrypt_generic_init(crypto_encrypt, crypto_key, crypto_key_len, iv))<0){
	forgetKey(0);
	free(iv);
	fprintf(stderr, "expandKey: ");
	mcrypt_perror(i);
	return "expandKey: Error initalizing encryption";
    }
    if((i=mcrypt_generic_init(crypto_decrypt, crypto_key, crypto_key_len, iv))<0){
	mcrypt_module_close(crypto_decrypt);
	crypto_decrypt=MCRYPT_FAILED;
	forgetKey(1);
	free(iv);
	fprintf(stderr, "expandKey: ");
	mcrypt_perror(i);
	return "expandKey: Error initalizing encryption";
    }
    free(iv);

    mcrypt_generic(crypto_encrypt, crypto_key, crypto_key_len);
    mcrypt_generic(crypto_encrypt, crypto_algo, crypto_algo_len);
    return NULL;
}

/* forget the key from the previous function */
void forgetKey(int i){
    if(crypto_encrypt!=MCRYPT_FAILED){
	if(i) mcrypt_generic_end(crypto_encrypt);
	else mcrypt_module_close(crypto_encrypt);
	crypto_encrypt=MCRYPT_FAILED;
    }
    if(crypto_decrypt!=MCRYPT_FAILED){
	if(i) mcrypt_generic_end(crypto_decrypt);
	else mcrypt_module_close(crypto_decrypt);
	crypto_decrypt=MCRYPT_FAILED;
    }
    if(crypto_key!=NULL){
	free(crypto_key);
	crypto_key=NULL;
    }
    if(crypto_algo!=NULL){
	free(crypto_algo);
	crypto_algo=NULL;
    }
}

/* Returns a crypter:           *\
|* You may encrypt and decrypt. *|
\* But NULL on error.           */
void *newCrypter(void *iv){
    MCRYPT *m;
    int i, e=errno;

    if((m=malloc(sizeof(MCRYPT)))==NULL){
	perror("newCrypter");
	return NULL;
    }

    mdecrypt_generic(crypto_decrypt, crypto_key, crypto_key_len);
    mdecrypt_generic(crypto_decrypt, crypto_algo, crypto_algo_len);
    *m=mcrypt_module_open(crypto_algo, NULL,
	   (mcrypt_module_is_block_algorithm(crypto_algo, NULL))?"cfb":"stream",
	   NULL);
    if(*m==MCRYPT_FAILED){
	fprintf(stderr, "DEBUG: %s\n", crypto_algo);
	mcrypt_generic(crypto_encrypt, crypto_key, crypto_key_len);
	mcrypt_generic(crypto_encrypt, crypto_algo, crypto_algo_len);
	free(m);
	fprintf(stderr, "newCrypter: mcrypt_module_open failed\n");
	return NULL; 
    }

    i=mcrypt_generic_init(*m, crypto_key, crypto_key_len, iv);
    mcrypt_generic(crypto_encrypt, crypto_key, crypto_key_len);
    mcrypt_generic(crypto_encrypt, crypto_algo, crypto_algo_len);
    if(i<0){
	mcrypt_module_close(*m);
	free(m);
	fprintf(stderr, "newCrypter: ");
	mcrypt_perror(i);
	return NULL;
    }
    errno=e;
    return m;
}

/* encrypt and decrypt strings */
int encryptString(void *crypter, void *plaintext, int len){
    return mcrypt_generic(*(MCRYPT *)crypter, plaintext, len);
}

int decryptString(void *crypter, void *plaintext, int len){
    return mdecrypt_generic(*(MCRYPT *)crypter, plaintext, len);
}

/* To close this crypter.            *\
|* Your memory, freed. Your pointer, *|
\* Invalidated.                      */
void freeCrypter(void *c){
    mcrypt_generic_end(*(MCRYPT *)c);
    free(c);
}

/* Allocates the buffer for an IV, but doesn't fill it in */
void *getIVBuf(int *len){
    void *iv;

    if(crypto_encrypt==MCRYPT_FAILED){
	errno=ENXIO;
	return NULL;
    }
    *len=mcrypt_enc_get_iv_size(crypto_encrypt);

    /* make sure we alloc at least 1 byte, so malloc can never return NULL */
    if((iv=malloc((*len)?*len:1))==NULL){
	return NULL;
    }
    return iv;
}

/* Generate a random IV. Length stored into len *\
|* Free it once you're done with it             *|
\* But NULL on error. errno set.                */
void *getIV(int *len){
    int fd;
    void *iv;

    iv=getIVBuf(len);
    if(*len==0) return iv;

    if((fd=open("/dev/random", O_RDONLY))<0){
	free(iv);
	return NULL;
    }

    if(read(fd, iv, *len)<*len){
	close(fd);
	free(iv);
	return NULL;
    }
    close(fd);
    return iv;
}
