#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <mcrypt.h>
#include <errno.h>

#include "pasmanedit.h"
#include "wmpasman.h"
#include "die.h"
#include "globals.h"
#include "secure.h"
#include "password.h"
#include "crypto.h"
#include "filestuff.h"

void usage(int exitcode) __attribute__ ((noreturn));

void usage(int exitcode){
    fprintf(stderr, "USAGE: %s [<options>] <command> <parameters>\n", PROGNAME);
    fprintf(stderr, "Options may be any of the following:\n");
    global_usage(20);
    fprintf(stderr, "\nCommands (and parameters) are:\n");
    fprintf(stderr, "- add  <label1> <label2> [<password>]\n    Add a new password entry. If <password> is omitted, you will be prompted.\n");
    fprintf(stderr, "- del  <label1> <label2>\n    Remove the (first) password with the given labels.\n");
#ifdef PASMANEDIT_LIST
    fprintf(stderr, "- list\n    List all passwords to stdout.\n");
#endif
    exit(exitcode);
}
    
int get_passphrase(void){
    char **algorithms;
    int numalgo, i, maxwidth, width;
    char *s;

    algorithms=mcrypt_list_algorithms(NULL, &numalgo);
    if(!algorithms) die("memory allocation");
    errno=0;
    if(numalgo<=0) die("libmcrypt reports no algorithms");
    maxwidth=0;
    for(i=0; i<numalgo; i++){
        width=strlen(algorithms[i]);
        if(maxwidth<width) maxwidth=width;
    }
    maxwidth+=4;

    width=0;
    printf("Available algorithms\n");
    printf("====================\n");
    for(i=0; i<numalgo; i++){
        printf("%-*s", maxwidth, algorithms[i]);
        width+=maxwidth;
        if(width+maxwidth>CONSOLE_WIDTH){
            printf("\n");
            width=0;
        }
    }
    if(width) printf("\n");
    printf("Choose Cypher: ");
    fflush(stdout);
    fgets(passphrase_buffer2, MAX_PHRASEHASH_LEN, stdin);
    s=passphrase_buffer2+strlen(passphrase_buffer2)-1;
    if(*s=='\n') *s=0;

    printf("Passphrase: ");
    fgets(password_buffer, MAX_PASSWORD_LEN, stdin);
    s=password_buffer+strlen(password_buffer)-1;
    if(*s=='\n') *s=0;

    if(crypto_set_passphrase(passphrase_buffer2, password_buffer)){
        sleep(5);
        debug(DEBUG_ERR, "Invalid cypher/passphrase");
        return -1;
    }
    return 0;
}

int main(int argc, char *argv[]){
    PWList list;
    Password p;
    int i;
    char *s, *t;

    if(init_globals(&argc, &argv)) usage(0);
    if(argc<2){
        debug(DEBUG_ERR, "No command specified!");
        usage(1);
    }

    if((list=pwlist_new(0))==NULL) die("memory allocation");
    while(get_passphrase());

    if((list=load_passwords())==NULL) die("Couldn't load passwords");

    switch(argv[1][0]){
      case 'a':
        if(argc!=4 && argc!=5){
            debug(DEBUG_ERR, "Invalid number of arguments for %s", argv[1]);
            usage(1);
        }
        if(argc==4){
            printf("Enter password: ");
            fflush(stdout);
            fgets(password_buffer, MAX_PASSWORD_LEN, stdin);
            s=password_buffer+strlen(password_buffer)-1;
            if(*s=='\n') *s=0;
            p=password_new(argv[2], argv[3], password_buffer);
        } else {
            p=password_new(argv[2], argv[3], argv[4]);
        }
        if(p==NULL) die("Couldn't store password");
        if(pwlist_push(list, p)) die("Couldn't store password");
        break;

      case 'd':
        if(argc!=4){
            debug(DEBUG_ERR, "Invalid number of arguments for %s", argv[1]);
            usage(1);
        }
        if(pwlist_count(list)<=0) die("No passwords");
        if((i=pwlist_find_i(list, argv[2], argv[3]))<0) die("Password \"%s\" \"%s\" not found", argv[2], argv[3]);
        pwlist_free_i(list, i);
        break;

#ifdef PASMANEDIT_LIST
      case 'l':
        if(argc!=2){
            debug(DEBUG_ERR, "Invalid number of arguments for %s", argv[1]);
            usage(1);
        }
        if(pwlist_count(list)<=0) die("No passwords");
        for(i=0; i<pwlist_count(list); i++){
            password_get_labels(list->p[i], (const char**)&s, (const char**)&t);
            printf("%-9s\t%-9s\t%s\n", s, t, password_get_pass(list->p[i]));
        }
        exit(0);
#endif /* def PASMANEDIT_LIST */

      default:
        debug(DEBUG_ERR, "Unknown command");
        usage(1);
    }

    if(save_passwords(list)) die("Couldn't save passwords");
    return 0;
}
