#include "config.h"

#include <gtk/gtk.h>

#include "clipboard.h"
#include "globals.h"
#include "die.h"

static GtkClipboard *clipboard=NULL, *primary=NULL;
static guint handler_id=-1;

void setup_selection(void){
    clipboard=gtk_clipboard_get(GDK_NONE);
    primary=gtk_clipboard_get(GDK_SELECTION_PRIMARY);

    if(!clipboard || !primary) die("Memory allocation");
}

static void clipboard_get(GtkClipboard *clip, GtkSelectionData *selection_data, guint info, gpointer d){
    Password pw;

    if(status==2){
        if((pw=pwlist_get_i(passwords, passwords_idx))!=NULL)
            gtk_selection_data_set_text(selection_data, password_get_pass(pw), -1);

        gtk_clipboard_clear(clipboard);
        gtk_clipboard_clear(primary);
    }
}

static void clipboard_clear(GtkClipboard *clip, gpointer user_data_or_owner){
    if(clip!=clipboard) gtk_clipboard_clear(clipboard);
    if(clip!=primary) gtk_clipboard_clear(primary);
    if(status==2) status=1;
}

static gboolean drop_clipboard(gpointer data){
    release_selection();
    return FALSE;
}

int get_selection(void){
    static const GtkTargetEntry targets[] = {
        { "STRING", 0, 1 },
        { "TEXT",   0, 2 }, 
        { "COMPOUND_TEXT", 0, 3 },
        { "UTF8_STRING", 0, 4 }
    };

    if(status<1){
        set_error("Not logged in");
        return -1;
    }
    if(status>2){
        set_error("No passwords");
        return -1;
    }

    if(status==1){
        if(!gtk_clipboard_set_with_data(primary, targets, G_N_ELEMENTS(targets), &clipboard_get, &clipboard_clear, NULL)){
            set_error("Failed to obtain primary selection");
            return -1;
        }
        if(!gtk_clipboard_set_with_data(clipboard, targets, G_N_ELEMENTS(targets), &clipboard_get, &clipboard_clear, NULL)){
            set_error("Failed to obtain clipboard selection");
            gtk_clipboard_clear(primary);
            return -1;
        }
    }

    if(handler_id!=-1) gtk_timeout_remove(handler_id);
    if(timeout!=-1)
        handler_id=gtk_timeout_add(timeout*1000, &drop_clipboard, NULL);
    status=2;
    return 0;
}

void release_selection(void){
    gtk_clipboard_clear(clipboard);
    gtk_clipboard_clear(primary);
    if(handler_id!=-1) gtk_timeout_remove(handler_id);
    if(status==2) status=1;
}
