#include "config.h"

#include <stdio.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "dock.h"
#include "clock.h"
#include "globals.h"
#include "wmgeneral/wmgeneral-gtk.h"

static char day_of_week[7][3] = { "SU", "MO", "TU", "WE", "TH", "FR", "SA" };
static char mon_of_year[12][4] = { "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
static char dday_of_week[5][3] = { "SM", "BT", "PD", "PP", "SO" };
static char dmon_of_year[5][4] = { "CHS", "DSC", "CFN", "BCY", "AFM" };

static void DrawTime(int hr, int min, int sec){
    char temp[16], *p;
    int i, j, x, y;
    char m;
    
    if(!(clock_flags&CLOCK_24HR)){
        m='A';
        if(hr>11){
            m='P';
            hr-=12;
        }
        if(hr==0) hr=12;
    } else {
        m='H';
    }
    y=(clock_flags&CLOCK_UTC)?84:74; /* Lowercase if UTC */

    snprintf(temp, 16, "%2d%02d%02d ", hr, min, sec);
    p=temp;
    x=5;
    for(i=0; i<3; i++){
        for(j=0; j<2; j++){
            if(*p==' ') *p='9'+1; /* space is one after the 9 */
            copyPixmapArea((*p-'0')*6, 64, 7, 9, x, 49);
            x+=6;
            p++;
        }
        x+=2;
    }

    copyPixmapArea((m-'A')*6, y, 7, 9, 51, 49);
}

static void DrawBeats(float b){
    char tmp[20];

    snprintf(tmp, 20, "@%07.3f ", b);
    DrawString(tmp, 50);
}


static void DrawDate(struct tm *t){
    char temp[16], *p;
    int i, x;

    if(clock_flags&CLOCK_DISCORDIAN){
        i=t->tm_yday;
        if(t->tm_year%4==0){
            if(t->tm_year%100==0 && t->tm_year%400!=100) goto NOPE;
            if(i==59){
                copyPixmapArea(60, 64, 7, 9, 5, 40);    /* space */
                copyPixmapArea(60, 64, 7, 9, 11, 40);   /* space */
                copyPixmapArea(108, 74, 13, 9, 23, 40); /* S */
                copyPixmapArea(114, 74, 7, 9, 29, 40);  /* T */
                copyPixmapArea(114, 74, 7, 9, 39, 40);  /* T */
                copyPixmapArea(48, 74, 7, 9, 45, 40);   /* I */
                copyPixmapArea(6, 74, 7, 9, 51, 40);    /* B */
                return;
            }
            if(i>59) i--;
        }
NOPE:
        snprintf(temp, 16, "%.2s%02d%.3s ", dday_of_week[i%5], (i%73)+1, dmon_of_year[i/73]);
    } else {
        snprintf(temp, 16, "%.2s%02d%.3s ", day_of_week[t->tm_wday], t->tm_mday, mon_of_year[t->tm_mon]);
    }

    x=5;
    p=temp;
    for(i=0; i<2; i++){
        copyPixmapArea((*p-'A')*6, 74, 7, 9, x, 40);
        x+=6;
        p++;
    }       

    x=23;
    for(i=0; i<2; i++){
        copyPixmapArea((*p-'0')*6, 64, 7, 9, x, 40);
        x+=6;
        p++;
    }

    x+=4;
    for(i=0; i<3; i++){
        copyPixmapArea((*p-'A')*6, 74, 7, 9, x, 40);
        x+=6;
        p++;
    }
}

int UpdateTime(void){
    static long nexttime=-1;
    static int old_flags=0;
    struct tm *tm=NULL;
    struct timeval tv;
    float b;
    int flag=0;

    if(old_flags!=clock_flags){
        copyPixmapArea(90, (clock_flags&CLOCK_BEATS)?15:5, 54, 9, 5, 49);
        nexttime=-1;
    }
    old_flags=clock_flags;

    gettimeofday(&tv, NULL);
    
    if(clock_flags&CLOCK_BEATS){
        if(gmtime(&tv.tv_sec)->tm_isdst<=0) tv.tv_sec+=3600;
        tm=gmtime(&tv.tv_sec);
        b=(tm->tm_hour*3600000+tm->tm_min*60000+tm->tm_sec*1000+(float)tv.tv_usec/1000)/86400;
        DrawBeats(b);
        flag=1;
    }

    if(tv.tv_sec>=nexttime){
        if(!flag){
            if(clock_flags&CLOCK_UTC){
                tm=gmtime(&tv.tv_sec);
            } else {
                tm=localtime(&tv.tv_sec);
            }
            DrawTime(tm->tm_hour, tm->tm_min, tm->tm_sec);
        }
        DrawDate(tm);
        flag=1;
    }
    
    nexttime=tv.tv_sec+1;
    return flag;
}
