#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "globals.h"
#include "die.h"

const char *error_string=NULL;

void set_error(char *fmt, ...){
    va_list ap;
    
    if(error_string) free((char *)error_string);
    va_start(ap, fmt);
    vasprintf((char **)&error_string, fmt, ap);
    va_end(ap);
    errno=-42;
}

static inline void print_stuff(char *fmt, va_list ap){
    fprintf(stderr, "%s: ", PROGNAME);
    vfprintf(stderr, fmt, ap);
    if(errno==-42){
        if(error_string) fprintf(stderr, ": %s", error_string);
        else fprintf(stderr, ": (Error allocating memory for error message)");
    } else if(errno){
        fprintf(stderr, ": %s", strerror(errno));
    }
    fprintf(stderr, "\n");
}

void xerrno(void){
    errno=0;
}

void debug(int level, char *fmt, ...){
    va_list ap;
    int e=0;

    if(debug_level<level) return;
    if(level>=DEBUG_BLEH){
        e=errno;
        errno=0;
    }
    va_start(ap, fmt);
    print_stuff(fmt, ap);
    va_end(ap);
    if(e!=0) errno=e;
}

void vdebug(int level, char *fmt, va_list ap){
    int e=0;

    if(debug_level<level) return;
    if(level>=DEBUG_BLEH){
        e=errno;
        errno=0;
    }
    print_stuff(fmt, ap);
    if(e!=0) errno=e;
}
   
void die(char *fmt, ...){
    va_list ap;

    va_start(ap, fmt);
    print_stuff(fmt, ap);
    va_end(ap);
    exit(1);
}

void vdie(char *fmt, va_list ap){
    print_stuff(fmt, ap);
    exit(1);
}
