#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "wmpasman.h"
#include "die.h"
#include "clipboard.h"
#include "dock.h"
#include "clock.h"
#include "popup_alert.h"
#include "popup_functions.h"
#include "popup_passphrase.h"
#include "popup_forgetpassphrase.h"
#include "popup_arrangeentries.h"

#include "wmgeneral/wmgeneral-gtk.h"
#include "wmgeneral/mouse_regions.h"
#include "wmgeneral/misc.h"

#include "xpm/wmpasman-master.xpm"
#include "xpm/wmpasman-mask.xbm"

void DrawString(const char *s, int y){
    int i, j;
    int x=5, a, b;
    char *chars="0123456789 :/\\!._@-[]()";

    for(i=0; i<9 && s[i]; i++){
        a=18;
        b=65;
        if(s[i]>='a' && s[i]<='z'){
            a=s[i]-'a';
            b=85;
        }
        if(s[i]>='A' && s[i]<='Z'){
            a=s[i]-'A';
            b=75;
        }                                                                               for(j=0; chars[j]; j++){
            if(chars[j]==s[i]){
                a=j;
                b=65;
                break;
            }
        }
        copyPixmapArea(a*6+1, b, 5, 7, x, y);
        x+=6;
    }
}

void redraw_dock(void){
    Password pw;
    const char *l1, *l2;

    copyPixmapArea(90, 16, 54, 7, 5, 20);
    copyPixmapArea(90, 16, 54, 7, 5, 28);
    if(status==0 || (pw=pwlist_get_i(passwords, passwords_idx))==NULL){
        if(status!=0) status=3;
    } else {
        if(status==3) status=1;
        password_get_labels(pw, &l1, &l2);
        DrawString(l1, 20);
        DrawString(l2, 28);
    }
    copyPixmapArea(status*6+91, 54, 4, 4, 51, 8);
    
    RedrawWindow();
}

void Function(GdkEventButton *ev){
    if(status==0){
        if(ev->button==3) return;
        GetPassphrase();
        return;
    }

    switch(ev->button){
      case 1:
        if(get_selection()) ErrAlert("Couldn't grap clipboard", "Couldn't grap clipboard");
        break;
      case 2:
        release_selection();
        break;
      case 3:
        release_selection();
        ForgetPassphrase();
        break;
    }
}

void RotateItem(GdkEventButton *ev){
    if(status==0 || pwlist_count(passwords)<=0) return;

    switch(ev->button){
      case 1:
      case 5:
        if(++passwords_idx>=pwlist_count(passwords)) passwords_idx=0;
        popup_arrange_select(passwords_idx);
        redraw_dock();
        break;
      case 3:
      case 4:
        if(--passwords_idx<0) passwords_idx=pwlist_count(passwords)-1;
        popup_arrange_select(passwords_idx);
        redraw_dock();
        break;
    }
}


void dock_click(GdkEventButton *ev){
    static int but_stat=-1;
    int i;

    if(ev->send_event) debug(DEBUG_BLEH, "Synthetic button %s event received", (ev->type==GDK_BUTTON_RELEASE)?"release":"press");
    switch(ev->type){
      case GDK_BUTTON_PRESS:
        but_stat=CheckMouseRegion(ev->x, ev->y);
        break;

      case GDK_BUTTON_RELEASE:
        i=CheckMouseRegion(ev->x, ev->y);
        if(but_stat==i && but_stat>=0){
            switch(but_stat){
              case 0:
                PopupDialog();
                break;
              case 1:
                Function(ev);
                break;
              case 2:
                RotateItem(ev);
                break;
              case 3:
                if(ev->button>0 && ev->button<8 && actions[ev->button-1]!=NULL)
                    execCommand(actions[ev->button-1]);
                break;
            }
            but_stat=-1;
        }
        break;

      default:
        /* Double click, or crap like that. Ignore. */
        break;
    }
}

gboolean dock_loop(gpointer data){
    static int oldstatus=-1;

    if(status==0 && passwords!=NULL){
        pwlist_free_all(passwords);
        passwords=NULL;
        changed=0;
    } else if(passwords==NULL){
        if(status!=0) status=0;
    } else if(pwlist_count(passwords)>0){
        if(status==3) status=1;
    } else {
        if(status>0) status=3;
    }
    if(UpdateTime() || status!=oldstatus) redraw_dock();
    oldstatus=status;
    return TRUE;
}

int dock_init(void){
    openDockWindow(0, NULL, wmpasman_master_xpm, wmpasman_mask_bits, wmpasman_mask_width, wmpasman_mask_height);
    setClickCallback(dock_click);

    AddMouseRegion(0,  5,  5, 41, 15);
    AddMouseRegion(1, 48,  5, 58, 15);
    AddMouseRegion(2,  5, 19, 58, 36);
    AddMouseRegion(3,  5, 40, 58, 58);

    gtk_timeout_add(250, dock_loop, NULL);

    return 0;
}
