#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "wmpasman.h"
#include "die.h"
#include "filestuff.h"
#include "globals.h"
#include "password.h"

char *make_path(char *path, char *file){
    char *str;
    
    asprintf(&str, "%s/%s", path, file);
    return str;
}

PWList load_passwords(void){
    PWList list;

    if((list=pwlist_new(0))==NULL) return NULL;
    if(pwlist_load(list, PWFILE)){
        pwlist_free_all(list);
        list=NULL;
    }
    return list;
}

int save_passwords(PWList list){
    char *file, *oldfile;
    int len, i;

    len=strlen(PWFILE)+7;
    if((file=malloc(len))==NULL) return -1;

    snprintf(file, len, "%s-new", PWFILE);
    unlink(file);
    if(pwlist_save(list, file)){
        unlink(file);
        free(file);
        return -1;
    }
    
    if(len<=MAX_PASSWORD_LEN){
        oldfile=password_buffer;
    } else {
        if((oldfile=malloc(len))==NULL){
            unlink(file);
            free(file);
            return -1;
        }
    }
    
    if(num_backups>99) num_backups=99;
    for(i=num_backups; i>0; i--){
        snprintf(file, len, "%s-bak%02d", PWFILE, i);
        if(i!=num_backups){
            if(unlink(oldfile) && errno!=ENOENT){
                set_error("Couldn't remove old backup file %s: %s", oldfile, strerror(errno));
                snprintf(file, len, "%s-new", PWFILE);
                unlink(file);
                free(file);
                if(oldfile!=password_buffer) free(oldfile);
                return -1;
            }
            if(link(file, oldfile) && errno!=ENOENT){
                set_error("Couldn't hardlink %s to %s: %s", oldfile, file, strerror(errno));
                snprintf(file, len, "%s-new", PWFILE);
                unlink(file);
                free(file);
                if(oldfile!=password_buffer) free(oldfile);
                return -1;
            }
        }
        strcpy(oldfile, file);
    }
    if(num_backups>0){
        strncpy(file, PWFILE, len);
        if(unlink(oldfile) && errno!=ENOENT){
            set_error("Couldn't remove old backup file %s: %s", oldfile, strerror(errno));
            snprintf(file, len, "%s-new", PWFILE);
            unlink(file);
            free(file);
            if(oldfile!=password_buffer) free(oldfile);
            return -1;
        }
        if(link(file, oldfile) && errno!=ENOENT){
            set_error("Couldn't hardlink %s to %s: %s", oldfile, file, strerror(errno));
            snprintf(file, len, "%s-new", PWFILE);
            unlink(file);
            free(file);
            if(oldfile!=password_buffer) free(oldfile);
            return -1;
        }
    }
    strncpy(oldfile, PWFILE, len);
    snprintf(file, len, "%s-new", PWFILE);
    if(unlink(oldfile) && errno!=ENOENT){
        set_error("Couldn't remove old backup file %s: %s", oldfile, strerror(errno));
        unlink(file);
        free(file);
        if(oldfile!=password_buffer) free(oldfile);
        return -1;
    }
    if(link(file, oldfile) && errno!=ENOENT){
        set_error("Couldn't hardlink %s to %s: %s", oldfile, file, strerror(errno));
        unlink(file);
        free(file);
        if(oldfile!=password_buffer) free(oldfile);
        return -1;
    }
    unlink(file);
    free(file);
    if(oldfile!=password_buffer) free(oldfile);
    return 0;
}
