#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>
#include <gtk/gtk.h>

#include "wmpasman.h"
#include "clock.h"
#include "die.h"
#include "password.h"
#include "secure.h"
#include "filestuff.h"

#include "wmgeneral/rcfile.h"

int debug_level=DEBUG_WARN;
int num_backups=NUM_BACKUPS;
char *PROGNAME=NULL;
char *CONF=NULL;
char *PWFILE=NULL;

char *password_buffer=NULL;
char *password_edit_buffer=NULL;
char *passphrase_buffer=NULL;
char *passphrase_buffer2=NULL;

int passwords_idx=0;
PWList passwords=NULL;
int status=0;
int changed=0;
int timeout=TIMEOUT;

GtkWindow *gtkwin=NULL;

int clock_flags=CLOCK_FLAGS;

char *actions[6];
rckeys wmpasman_keys[]={
    { "num-backups", NULL },
    { "timeout", NULL },
    { "beats", NULL },
    { "24hr", NULL },
    { "utc", NULL },
    { "discordian", NULL }, { "ddate", NULL },
    { "pwfile", &PWFILE },
    { "button1", &actions[0] }, { "left", &actions[0] },
    { "button2", &actions[1] }, { "middle", &actions[1] },
    { "button3", &actions[2] }, { "right", &actions[2] },
    { "button4", &actions[3] }, { "wheel-up", &actions[3] },
    { "button5", &actions[4] }, { "wheel-down", &actions[4] },
    { "button6", &actions[5] },
    { "button7", &actions[6] },
    { NULL, NULL }
};

void global_usage(int col){
    fprintf(stderr, "%-*sConfiguration file to use\n", col, "  -c, --conf <file>  ");
    fprintf(stderr, "%-*sPassword file to use\n", col, "  -p, --pwfile <file>  ");
    fprintf(stderr, "%-*sNumber of password file backups to keep\n", col, "  -n, --num_backups <n>  ");
    fprintf(stderr, "%-*sThis message\n", col, "      --help  ");
    fprintf(stderr, "%-*sPrint the version number and exit\n", col, "  -V, --version  ");
    fprintf(stderr, "%-*sPrint more messages (can be repeated)\n", col, "  -v, --verbose  ");
    fprintf(stderr, "%-*sPrint fewer messages (can be repeated)\n", col, "  -q, --quiet  ");
}

int init_globals(int *argc, char ***argv){
    int i, j;
    char *HOME=NULL, *c;
    struct passwd *pw=NULL;

#define ARG(i, short, long) ((short!=NULL && !strcmp((*argv)[i], short)) || (long!=NULL && !strcmp((*argv)[i], long)))

    if((PROGNAME=strrchr((*argv)[0], '/'))==NULL || !PROGNAME[1])
        PROGNAME=(*argv)[0];
    else PROGNAME++;
    
    /* WARNING: We may still be SUID here, so don't do anything insecure! */
    /* Parse command line args we need for setup here */
    for(i=1, j=1; i<*argc; i++){
        if(ARG(i, "-v", "--verbose")){
            debug_level++;
        } else if(ARG(i, "-q", "--quiet")){
            debug_level--; if(debug_level<0) debug_level=0;
        } else if(ARG(i, "-V", "--version")){
            xerrno();
            die("%s", VERSION);
        } else if(ARG(i, NULL, "--help")){
            return 1;
        } else if(ARG(i, "-c", "--conf")){
            if(i+1>=*argc) die("%s requires an argument", (*argv)[i]);
            CONF=(*argv)[++i];
        } else {
            (*argv)[j++]=(*argv)[i];
        }
    }
    *argc=j;

    secure_startup();

    /* OK, no longer SUID. Do nasty stuff now. */
    /* Get path to config file, if we don't already have it */
    HOME=getenv("HOME");
    if(HOME==NULL){
        pw=getpwuid(getuid());
        if(pw==NULL || pw->pw_dir==NULL){
            debug(DEBUG_ERR, "$HOME not set and getpwuid(%d) failed", getuid());
            xerrno();
            die("Please set $HOME, or use the --conf and --pwfile options.");
        }
        HOME=pw->pw_dir;
    }
    if(!CONF) CONF=make_path(HOME, ".wmpasmanrc");
    debug(DEBUG_BLEH, "Conf file is %s", CONF);

    /* Read config file */
    {
        char *opts[6];
        for(i=0; i<sizeof(opts)/sizeof(opts[0]); i++){
            opts[i]=NULL;
            wmpasman_keys[i].var=&opts[i];
        }
        wmpasman_keys[6].var=&opts[5];
        for(i=0; i<sizeof(actions)/sizeof(actions[0]); actions[i++]=NULL);
        parse_rcfile("/etc/wmpasmanrc", wmpasman_keys);
        parse_rcfile(CONF, wmpasman_keys);
        if(opts[0]!=NULL){ /* num-backups */
            num_backups=strtol(opts[0], &c, 0);
            if(c!=NULL && (c==opts[0] || *c!='\0')) die("%s requires a numeric argument", wmpasman_keys[0].label);
            if(num_backups<0) die("Cannot make negative backups");
            xerrno();
            if(num_backups==0) debug(DEBUG_INFO, "Password file backups disabled!");
            else debug(DEBUG_BLEH, "Will maintain %d levels of backup", num_backups);
        }
        if(opts[1]!=NULL){ /* timeout */
            timeout=strtol(opts[1], &c, 0);
            if(c!=NULL && (c==opts[1] || *c!='\0')) die("%s requires a numeric argument", wmpasman_keys[1].label);
            if(timeout<=0 && timeout!=-1) die("Timeout must be greater than 0 (or -1)");
            if(timeout==-1) debug(DEBUG_BLEH, "No timeout");
            else debug(DEBUG_BLEH, "Timeout after %d seconds", timeout);
        }
        if(opts[2]!=NULL){ /* beats */
            clock_flags^=CLOCK_BEATS;
            debug(DEBUG_BLEH, "Beats mode %s", (clock_flags&CLOCK_BEATS)?"on":"off");
        }
        if(opts[3]!=NULL){ /* 24hr */
            clock_flags^=CLOCK_24HR;
            debug(DEBUG_BLEH, "24-hour mode %s", (clock_flags&CLOCK_24HR)?"on":"off");
        }
        if(opts[4]!=NULL){ /* utc */
            clock_flags^=CLOCK_UTC;
            debug(DEBUG_BLEH, "UTC mode %s", (clock_flags&CLOCK_UTC)?"on":"off");
        }
        if(opts[5]!=NULL){ /* ddate */
            clock_flags^=CLOCK_DISCORDIAN;
            debug(DEBUG_BLEH, "Discordian mode %s", (clock_flags&CLOCK_DISCORDIAN)?"on":"off");
        }
    }

    /* Parse remaining command line arguments */
    for(i=1, j=1; i<*argc; i++){
        if(ARG(i, "-n", "--num-backups")){
            if(i+1>=*argc) die("%s requires a numeric argument", (*argv)[i]);
            num_backups=strtol((*argv)[i+1], &c, 0);
            if(c!=NULL && (c==(*argv)[i+1] || *c!='\0')) die("%s requires a numeric argument", (*argv)[i]);
            if(num_backups<0) die("Cannot make negative backups");
            xerrno();
            if(num_backups==0) debug(DEBUG_INFO, "Password file backups disabled!");
            else debug(DEBUG_BLEH, "Will maintain %d levels of backup", num_backups);
            i++;
        } else if(ARG(i, "-p", "--pwfile")){
            if(i+1>=*argc) die("%s requires an argument", (*argv)[i]);
            PWFILE=(*argv)[++i];
        } else {
            (*argv)[j++]=(*argv)[i];
        }
    }
    *argc=j;

    if(!PWFILE) PWFILE=make_path(HOME, ".wmpasman");
    if(pw!=NULL) free(pw);
    passwords=pwlist_new(0);
    if(!passwords || !CONF || !PWFILE) die("Memory allocation");
    debug(DEBUG_BLEH, "Password file is %s", PWFILE);
    return 0;
}
