#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <signal.h>
#include <gtk/gtk.h>

#include "wmpasman.h"
#include "globals.h"
#include "clipboard.h"
#include "crypto.h"
#include "secure.h"
#include "die.h"
#include "dock.h"
#include "clock.h"

RETSIGTYPE sigchld(int i){
    waitpid(-1, NULL, WNOHANG);
    if(signal(SIGCHLD, sigchld)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting SIGCHLD handler");
}

RETSIGTYPE SwapBeats(int i){
    clock_flags^=CLOCK_BEATS;
    debug(DEBUG_BLEH, "Beats mode %s", (clock_flags&CLOCK_BEATS)?"on":"off");
    if(signal(SIGBEATS, SwapBeats)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for Beats toggle", SIGBEATS);
}

RETSIGTYPE SwapDDate(int i){
    clock_flags^=CLOCK_DISCORDIAN;
    debug(DEBUG_BLEH, "Discordian mode %s", (clock_flags&CLOCK_DISCORDIAN)?"on":"off");
    if(signal(SIGDDATE, SwapDDate)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for Dischordian toggle", SIGDDATE);
}

RETSIGTYPE SwapUTC(int i){
    clock_flags^=CLOCK_UTC;
    debug(DEBUG_BLEH, "UTC mode %s", (clock_flags&CLOCK_UTC)?"on":"off");
    if(signal(SIGUTC, SwapUTC)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for UTC toggle", SIGUTC);
}

RETSIGTYPE Swap24hr(int i){
    clock_flags^=CLOCK_24HR;
    debug(DEBUG_BLEH, "24-hour mode %s", (clock_flags&CLOCK_24HR)?"on":"off");
    if(signal(SIG24HR, Swap24hr)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for 24-hour toggle", SIG24HR);
}

void usage(int exitcode) __attribute__ ((noreturn));

void usage(int exitcode){
    fprintf(stderr, "USAGE: %s [<options>]\n", PROGNAME);
    fprintf(stderr, "Options may be any of the standard Gtk+ options, as well as:\n");
    fprintf(stderr, "  -B, --beats      Use beats <http://www.timeanddate.com/time/internettime.html>\n");
    fprintf(stderr, "  -M, --24hr       Display time in 24-hour (\"military\") format\n");
    fprintf(stderr, "  -U, --utc        Display time in UTC instead of the local timezone\n");
    fprintf(stderr, "  -D, --ddate      Dischordian mode (consult your pineal gland)\n");
    fprintf(stderr, "      --discordian Dischordian mode (consult your pineal gland)\n");
    fprintf(stderr, "  -t, --timeout <seconds> Time to hold the clipboard\n");
    fprintf(stderr, "                   (Set to -1 to disable)\n");
    global_usage(20);
    fprintf(stderr, "\n");
    exit(exitcode);
}

int main(int argc, char *argv[]){
    int i;
    
    if(init_globals(&argc, &argv)) usage(0);
    gtk_init(&argc, &argv);

    while(1){
        static struct option long_options[]={
            {"beats", 0, NULL, 'B'},
            {"ddate", 0, NULL, 'D'},
            {"discordian", 0, NULL, 'D'},
            {"utc", 0, NULL, 'U'},
            {"24hr", 0, NULL, 'M'},
            {"timeout", 0, NULL, 't'},
            {0,0,0,0}
        };

        i=getopt_long(argc, argv, "BDgUMt:", long_options, NULL);
        if(i==-1) break;

        switch(i){
          case 'B':
            SwapBeats(0);
            break;

          case 'D':
            SwapDDate(0);
            break;

          case 'g':
            debug(DEBUG_WARN, "-g is depreciated, please use -U/--utc instead");
            /* fall through */
          case 'U':
            SwapUTC(0);
            break;

          case 'M':
            Swap24hr(0);
            break;

          case 't':
            timeout=atoi(optarg);
            if(timeout<=0 && timeout!=-1) die("Timeout must be greater than 0 (or -1)");
            if(timeout==-1) debug(DEBUG_BLEH, "No timeout");
            else debug(DEBUG_BLEH, "Timeout after %d seconds", timeout);
            break;

          case ':':
            die("Missing parameter to %s", argv[optind]);

          case '?':
            debug(DEBUG_ERR, "Unknown option %s", argv[optind]);
            usage(1);

          default:
            die("WTF? getopt_long returned %d", i);
        }
    }
    if(optind<argc) die("Unknown parameter %s", argv[i]);

    if(signal(SIGBEATS, SwapBeats)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for Beats toggle", SIGBEATS);
    if(signal(SIG24HR, Swap24hr)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for 24-hour toggle", SIG24HR);
    if(signal(SIGUTC, SwapUTC)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for UTC toggle", SIGUTC);
    if(signal(SIGDDATE, SwapDDate)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting signal %d for Dischordian toggle", SIGDDATE);
    if(signal(SIGCHLD, sigchld)==SIG_ERR)
        debug(DEBUG_WARN, "Error setting SIGCHLD handler");

    dock_init();
    setup_selection();
    gtk_main();
    return 0;
}
