#include "config.h"

#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>

#include "globals.h"

static GtkWidget *dialog;

extern const char *error_string;

void ErrAlert(char *title, char *msg){
    if(dialog!=NULL) gtk_widget_destroy(dialog);
    
    if(errno==-42){
        dialog=gtk_message_dialog_new(gtkwin, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s: %s", msg, error_string);
    } else if(errno){
        dialog=gtk_message_dialog_new(gtkwin, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s: %s", msg, strerror(errno));
    } else {
        dialog=gtk_message_dialog_new(gtkwin, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", msg);
    }
    gtk_window_set_title(GTK_WINDOW(dialog), title);
    g_signal_connect_swapped(GTK_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(dialog));
    g_signal_connect(G_OBJECT(dialog), "destroy", G_CALLBACK(gtk_widget_destroyed), &dialog);

    gtk_widget_show_all(dialog);
}

void Alert(GtkMessageType type, char *title, char *msg){
    if(dialog!=NULL) gtk_widget_destroy(dialog);
    
    dialog=gtk_message_dialog_new(gtkwin, GTK_DIALOG_MODAL, type, GTK_BUTTONS_CLOSE, "%s", msg);
    gtk_window_set_title(GTK_WINDOW(dialog), title);
    g_signal_connect_swapped(GTK_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(dialog));
    g_signal_connect(G_OBJECT(dialog), "destroy", G_CALLBACK(gtk_widget_destroyed), &dialog);

    gtk_widget_show_all(dialog);
}

void YesNo(char *title, char *msg, void (*callback)(GtkDialog *dialog, gint arg1, gpointer d), gpointer data){
    if(dialog!=NULL) gtk_widget_destroy(dialog);
    
    dialog=gtk_message_dialog_new(gtkwin, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s", msg);
    gtk_window_set_title(GTK_WINDOW(dialog), title);
    gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
    if(callback!=NULL) g_signal_connect(GTK_OBJECT(dialog), "response", G_CALLBACK(callback), data);
    g_signal_connect_swapped(GTK_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(dialog));
    g_signal_connect(G_OBJECT(dialog), "destroy", G_CALLBACK(gtk_widget_destroyed), &dialog);

    gtk_widget_show_all(dialog);
}
