#include "config.h"

#include <string.h>
#include <gtk/gtk.h>

#include "dock.h"
#include "globals.h"
#include "die.h"
#include "password.h"
#include "popup_arrangeentries.h"
#include "popup_alert.h"

GtkTreeSelection *sel=NULL;

enum {
    LINE1_COL,
    LINE2_COL,
    PASSWORD_COL,
    N_COLS
};

static void list_store_set(GtkListStore *store, GtkTreeIter *iter, Password pw){
    const char *l1, *l2;

    password_get_labels(pw, &l1, &l2);
    gtk_list_store_set(GTK_LIST_STORE(store), iter,
                       LINE1_COL, l1,
                       LINE2_COL, l2,
                       PASSWORD_COL, pw,
                       -1);
}

/* Set the columns in view to indicate that the thing is sorted by 'col',
 * and return the direction of the sort */
static GtkSortType set_sort(GtkTreeView *view, int col){
    GList *cols;
    GtkTreeViewColumn *cur;
    GtkSortType sort=GTK_SORT_ASCENDING;
    int i;

    cols=gtk_tree_view_get_columns(view);
    for(i=0; i<g_list_length(cols); i++){
        cur=GTK_TREE_VIEW_COLUMN(g_list_nth_data(cols, i));
        if(col==GPOINTER_TO_INT(g_object_get_data(G_OBJECT(cur), "col"))){
            if(gtk_tree_view_column_get_sort_indicator(cur)){
                sort=(gtk_tree_view_column_get_sort_order(cur)==GTK_SORT_ASCENDING)?GTK_SORT_DESCENDING:GTK_SORT_ASCENDING;
            }
            gtk_tree_view_column_set_sort_indicator(cur, TRUE);
            gtk_tree_view_column_set_sort_order(cur, sort);
        } else {
            gtk_tree_view_column_set_sort_indicator(cur, FALSE);
        }
    }
    g_list_free(cols);
    return sort;
}

/* DND end handler, signal that rows were reordered */
static void drag_end(GtkTreeView *view, gpointer x, GtkTreeModel *store){
    GtkTreePath *path;
    gint i;

    set_sort(view, -1);
    path=gtk_tree_path_new_first();
    gtk_tree_model_rows_reordered(store, path, NULL, &i);
    gtk_tree_path_free(path);
}

struct SortTuple {
    const gchar *label;
    Password pw;
};

/* Sort SortTuples by label */
static gint sort_func(gconstpointer a, gconstpointer b, gpointer dir){
    int result;

    result=strcasecmp(((struct SortTuple *)a)->label, ((struct SortTuple *)b)->label);
    if(GPOINTER_TO_INT(dir)==GTK_SORT_ASCENDING) return result;
    return -result;
}

/* Sort by the specified column */
static void sort_by_col(GObject *c, GtkTreeView *view){
    GtkTreeModel *store;
    GtkSortType sort;
    GtkTreeIter iter;
    Password selected, current, pw;
    int col, i;
    GArray *sort_array;

    /* Get the selected item, column number, sort direction, and data store */
    if(gtk_tree_selection_get_selected(sel, &store, &iter)){
        gtk_tree_model_get(store, &iter, PASSWORD_COL, &selected, -1);
    } else {
        selected=NULL;
    }
    current=passwords->p[passwords_idx];

    col=GPOINTER_TO_INT(g_object_get_data(c, "col"));
    sort=set_sort(view, col);

    /* Fill in the sort array */
    sort_array=g_array_sized_new(FALSE, FALSE, sizeof(struct SortTuple), passwords->num);
    gtk_tree_model_get_iter_first(store, &iter);
    i=0;
    do {
        struct SortTuple tuple;
        gtk_tree_model_get(store, &iter, col, &tuple.label, PASSWORD_COL, &tuple.pw, -1);
        g_array_append_val(sort_array, tuple);
        i++;
    } while(gtk_tree_model_iter_next(store, &iter));
    if(i!=passwords->num) die("WTF? %d != %d", i, passwords->num);

    /* Sort the array */
    g_array_sort_with_data(sort_array, sort_func, GINT_TO_POINTER(sort));

    /* Pull data out of the sorted array, and update both the PWList and the
     * GtkListStore */
    gtk_tree_model_get_iter_first(store, &iter);
    i=0;
    do {
        pw=g_array_index(sort_array, struct SortTuple, i).pw;
        list_store_set(GTK_LIST_STORE(store), &iter, pw);
        passwords->p[i]=pw;
        if(pw==selected) gtk_tree_selection_select_iter(sel, &iter);
        if(pw==current) passwords_idx=i;
        i++;
    } while(gtk_tree_model_iter_next(store, &iter));
    popup_arrange_select(passwords_idx);
    changed=1;
    redraw_dock();

    /* Done */
    g_array_free(sort_array, TRUE);
}

static void rows_reordered(GtkTreeModel *store, gpointer null1, gpointer null2, gint *new_order, gpointer data){
    int i;
    GtkTreeIter iter;
    Password current;

    current=passwords->p[passwords_idx];
    gtk_tree_model_get_iter_first(store, &iter);
    i=0;
    do {
        gtk_tree_model_get(store, &iter, PASSWORD_COL, &(passwords->p[i]), -1);
        if(passwords->p[i]==current) passwords_idx=i;
        i++;
    } while(gtk_tree_model_iter_next(store, &iter));
    if(i!=passwords->num) die("WTF? %d != %d", i, passwords->num);
    popup_arrange_select(passwords_idx);
    changed=1;
    redraw_dock();
}

static void selection_changed_1(GtkTreeSelection *sel, GtkWidget *b){
    GtkTreeModel *store;
    GtkTreeIter iter;
    Password pw;

    if(gtk_tree_selection_get_selected(sel, &store, &iter)){
        gtk_tree_model_get(store, &iter, PASSWORD_COL, &pw, -1);
        passwords_idx=pwlist_get_p(passwords, pw);
        redraw_dock();
        if(passwords_idx!=0){
            gtk_widget_set_sensitive(b, TRUE);
            return;
        }
    }
    gtk_widget_set_sensitive(b, FALSE);
}

static void selection_changed_2(GtkTreeSelection *sel, GtkWidget *b){
    GtkTreeModel *store;
    GtkTreeIter iter;

    if(gtk_tree_selection_get_selected(sel, &store, &iter)){
        if(gtk_tree_model_iter_next(store, &iter)){
            gtk_widget_set_sensitive(b, TRUE);
            return;
        }
    }
    gtk_widget_set_sensitive(b, FALSE);
}

static void move_button(GtkButton *b, gpointer dir){
    GtkTreeModel *store;
    GtkTreeIter iter, dest;
    GtkTreePath *path;
    Password pw1, pw2;
    int i;

    if(!gtk_tree_selection_get_selected(sel, &store, &iter)) return;
    path=gtk_tree_model_get_path(store, &iter);
    if(GPOINTER_TO_INT(dir)<0){
        if(!gtk_tree_path_prev(path)) /* already at the top */
            goto DONE;
        if(!gtk_tree_model_get_iter(store, &dest, path)){ /* wtf? */
            debug(DEBUG_INFO, "Odd, gtk_tree_path_prev succeeded but gtk_tree_model_get_iter failed...");
            goto DONE;
        }
    } else {
        dest=iter;
        if(!gtk_tree_model_iter_next(store, &dest)) /* already at the end */
            goto DONE;
        gtk_tree_path_next(path);
    }
    gtk_tree_model_get(store, &iter, PASSWORD_COL, &pw1, -1);
    gtk_tree_model_get(store, &dest, PASSWORD_COL, &pw2, -1);
    list_store_set(GTK_LIST_STORE(store), &dest, pw1);
    list_store_set(GTK_LIST_STORE(store), &iter, pw2);
    gtk_tree_model_rows_reordered(store, path, NULL, &i);
DONE:
    gtk_tree_path_free(path);
}

#define DAMNIT() { if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin)); debug(DEBUG_ERR, "Couldn't create GTK widgets"); return; }

void ArrangeEntries(GtkObject *b, gpointer data){
    GtkBox *vbox, *box;
    GtkListStore *store;
    GtkTreeIter iter;
    GtkTreePath *selpath=NULL;
    GtkTreeView *list;
    GtkScrolledWindow *scrolled;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkWidget *w, *w2;
    int i;

    if(status==0) return;
    if(passwords->num<=0){
        Alert(GTK_MESSAGE_INFO, "No passwords!", "There are no passwords to arrange.");
        return;
    }

    if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin));
    gtkwin=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
    if(gtkwin==NULL) DAMNIT();
    gtk_window_set_title(GTK_WINDOW(gtkwin), "Arrange Password Entries");
    gtk_container_set_border_width(GTK_CONTAINER(gtkwin), 5);
    g_signal_connect(G_OBJECT(gtkwin), "destroy", G_CALLBACK(gtk_widget_destroyed), &gtkwin);
    g_signal_connect(G_OBJECT(gtkwin), "destroy", G_CALLBACK(gtk_widget_destroyed), &sel);

    vbox=GTK_BOX(gtk_vbox_new(0,2));
    if(vbox==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(gtkwin), GTK_WIDGET(vbox));

    box=GTK_BOX(gtk_hbox_new(0,2));
    if(box==NULL) DAMNIT();
    gtk_box_pack_start(vbox, GTK_WIDGET(box), 1, 1, 0);

    store=gtk_list_store_new(N_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
    if(store==NULL) DAMNIT();
    list=GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(store)));
    if(list==NULL) DAMNIT();
    sel=gtk_tree_view_get_selection(list);
    if(sel==NULL) DAMNIT();
    gtk_tree_selection_set_mode(sel, GTK_SELECTION_BROWSE);

    for(i=0; i<passwords->num; i++){
        gtk_list_store_append(store, &iter);
        list_store_set(store, &iter, passwords->p[i]);
        if(i==passwords_idx) selpath=gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
    }

    if((renderer=gtk_cell_renderer_text_new())==NULL) DAMNIT();
    column=gtk_tree_view_column_new_with_attributes("Label 1",
                                                    renderer,
                                                    "text", LINE1_COL,
                                                    NULL);
    if(column==NULL) DAMNIT();
    g_object_set_data(G_OBJECT(column), "col", GINT_TO_POINTER(0));
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_clickable(column, TRUE);
    g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_by_col), list);
    gtk_tree_view_append_column(list, column);
    column=gtk_tree_view_column_new_with_attributes("Label 2",
                                                    renderer,
                                                    "text", LINE2_COL,
                                                    NULL);
    if(column==NULL) DAMNIT();
    g_object_set_data(G_OBJECT(column), "col", GINT_TO_POINTER(1));
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_clickable(column, TRUE);
    g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_by_col), list);
    gtk_tree_view_append_column(list, column);
    gtk_tree_view_set_reorderable(list, TRUE);
    g_signal_connect(G_OBJECT(store), "rows_reordered", G_CALLBACK(rows_reordered), NULL);
    g_signal_connect(G_OBJECT(list), "drag_end", G_CALLBACK(drag_end), store);

    scrolled=GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
    if(scrolled==NULL) DAMNIT();
    gtk_scrolled_window_set_policy(scrolled, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(scrolled, GTK_SHADOW_IN);
    gtk_widget_set_size_request(GTK_WIDGET(scrolled), -1, 150);
    gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(list));
    gtk_box_pack_start(box, GTK_WIDGET(scrolled), 1, 1, 5);
    
    w=gtk_vbox_new(0,5);
    w2=gtk_alignment_new(0.5, 0.5, 0, 0);
    if(w==NULL || w2==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(w2), w);
    gtk_box_pack_start(box, w2, 1, 1, 0);
    box=GTK_BOX(w);
    
    w=gtk_button_new();
    w2=gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_OUT);
    if(w==NULL || w2==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(w), w2);
    gtk_box_pack_start(box, w, 0, 0, 0);
    g_signal_connect(w, "clicked", G_CALLBACK(move_button), GINT_TO_POINTER(-1));
    g_signal_connect(sel, "changed", G_CALLBACK(selection_changed_1), w);

    w=gtk_button_new();
    w2=gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
    if(w==NULL || w2==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(w), w2);
    gtk_box_pack_start(box, w, 0, 0, 0);
    g_signal_connect(w, "clicked", G_CALLBACK(move_button), GINT_TO_POINTER(1));
    g_signal_connect(sel, "changed", G_CALLBACK(selection_changed_2), w);

    w=gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_box_pack_start(vbox, w, 0, 0, 0);
    g_signal_connect_swapped(G_OBJECT(w), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);

    gtk_widget_show_all(GTK_WIDGET(gtkwin));

    if(selpath){
        gtk_tree_view_set_cursor(list, selpath, NULL, FALSE);
        gtk_widget_grab_focus(GTK_WIDGET(list));
        gtk_tree_path_free(selpath);
    }
}

void popup_arrange_select(int idx){
    GtkTreeIter iter;
    GtkTreePath *path;
    GtkTreeView *view;
    GtkTreeModel *store;

    if(sel==NULL) return;
    view=gtk_tree_selection_get_tree_view(sel);
    store=gtk_tree_view_get_model(view);
    if(!gtk_tree_model_iter_nth_child(store, &iter, NULL, passwords_idx)){
        debug(DEBUG_WARN, "Something claimed password number %d was selected, but i have only %d...", passwords_idx, gtk_tree_model_iter_n_children(store, NULL));
        return;
    }
    path=gtk_tree_model_get_path(store, &iter);
    gtk_tree_view_set_cursor(view, path, NULL, FALSE);
    gtk_tree_path_free(path);
}
