#include "config.h"

#include <string.h>
#include <ctype.h>
#include <gtk/gtk.h>

#include "wmpasman.h"
#include "dock.h"
#include "globals.h"
#include "die.h"
#include "password.h"
#include "popup_editentry.h"
#include "popup_alert.h"
#include "gtk_mytextbuffer.h"

/* Signal handler to toggle the visibility of the GtkTextBuffer */
static void toggle_visibility(GtkToggleButton *t, GtkTextBuffer *b){
    my_text_buffer_set_visibility(b, gtk_toggle_button_get_active(t));
}

static void clicked(GObject *b, Password oldpw){
    GtkEditable *e1, *e2;
    GtkTextBuffer *buffer;
    Password pw;
    int i;

    i=pwlist_get_p(passwords, oldpw);
    if(i<0){
        Alert(GTK_MESSAGE_ERROR, "Error editing password", "Couldn't find the old password object to replace it!");
        return;
    }

    e1=GTK_EDITABLE(g_object_get_data(b, "e1"));
    e2=GTK_EDITABLE(g_object_get_data(b, "e2"));
    buffer=GTK_TEXT_BUFFER(g_object_get_data(b, "buffer"));
    pw=password_new(gtk_editable_get_chars(e1, 0, -1), gtk_editable_get_chars(e2, 0, -1), my_text_buffer_get_buffer(buffer));
    if(pw==NULL){
        ErrAlert("Error editing password", "Couldn't create a new Password object");
        return;
    }

    /* Cheat, and swap the old password for the new directly. That's why PWList
     * is a publicly defined struct. */
    passwords->p[i]=pw;
    password_free(oldpw);

    redraw_dock();
}

#define DAMNIT() { if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin)); debug(DEBUG_ERR, "Couldn't create GTK widgets"); return; }

void EditEntry(GtkObject *b, Password pw){
    GtkBox *vbox, *hbox;
    GtkTable *table;
    GtkEntry *e1, *e2;
    GtkTextBuffer *buffer;
    GtkWidget *w, *checkbutton;
    GtkScrolledWindow *scrolled;
    const char *l1, *l2;
    
    if(status==0) return;

    if(pw==NULL && (pw=pwlist_get_i(passwords, passwords_idx))==NULL){
        Alert(GTK_MESSAGE_ERROR, "No passwords to edit", "There are no passwords to edit.");
        return;
    }

    if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin));

    gtkwin=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
    if(gtkwin==NULL) DAMNIT();
    gtk_window_set_title(GTK_WINDOW(gtkwin), "Edit a Password Entry");
    gtk_container_set_border_width(GTK_CONTAINER(gtkwin), 5);
    g_signal_connect(G_OBJECT(gtkwin), "destroy", G_CALLBACK(gtk_widget_destroyed), &gtkwin);

    vbox=GTK_BOX(gtk_vbox_new(0,2));
    if(vbox==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(gtkwin), GTK_WIDGET(vbox));

    table=GTK_TABLE(gtk_table_new(2, 2, 0));
    if(table==NULL) DAMNIT();
    gtk_box_pack_start(vbox, GTK_WIDGET(table), 0, 0, 0);

    w=gtk_label_new("Password Label:");
    gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
    gtk_table_attach(table, w, 0,1, 0,2, GTK_FILL|GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

    password_get_labels(pw, &l1, &l2);
    e1=GTK_ENTRY(gtk_entry_new());
    if(e1==NULL) DAMNIT();
    gtk_entry_set_max_length(e1, LINE_CHARS);
    gtk_entry_set_text(e1, l1);
    gtk_editable_set_position(GTK_EDITABLE(e1), -1);
    gtk_table_attach(table, GTK_WIDGET(e1), 1,2, 0,1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 5, 0);

    e2=GTK_ENTRY(gtk_entry_new());
    if(e2==NULL) DAMNIT();
    gtk_entry_set_max_length(e2, LINE_CHARS);
    gtk_entry_set_text(e2, l2);
    gtk_editable_set_position(GTK_EDITABLE(e2), -1);
    gtk_table_attach(table, GTK_WIDGET(e2), 1,2, 1,2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 5, 0);

    hbox=GTK_BOX(gtk_hbox_new(0,2));
    w=gtk_alignment_new(0.0, 0.5, 0, 0);
    if(hbox==NULL || w==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(w), GTK_WIDGET(hbox));
    gtk_box_pack_start(vbox, w, 0, 0, 0);

    w=gtk_label_new("Password:");
    gtk_box_pack_start(hbox, w, 0, 0, 0);

    if((checkbutton=gtk_check_button_new())==NULL) DAMNIT();
    gtk_box_pack_start(hbox, checkbutton, 0, 0, 0);
    
    buffer=my_text_buffer_new(password_edit_buffer, MAX_PASSWORD_LEN);
    w=gtk_text_view_new_with_buffer(buffer);
    gtk_text_buffer_insert_at_cursor(buffer, password_get_pass(pw), -1);

    scrolled=GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
    gtk_scrolled_window_set_policy(scrolled, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(scrolled, GTK_SHADOW_IN);
    gtk_container_add(GTK_CONTAINER(scrolled), w);
    gtk_box_pack_start(vbox, GTK_WIDGET(scrolled), 1, 1, 5);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), my_text_buffer_get_visibility(buffer));
    g_signal_connect(G_OBJECT(checkbutton), "toggled", G_CALLBACK(toggle_visibility), buffer);

    hbox=GTK_BOX(gtk_hbutton_box_new());
    if(hbox==NULL) DAMNIT();
    gtk_box_pack_start(vbox, GTK_WIDGET(hbox), 0, 0, 0);

    w=gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_box_pack_start(hbox, w, 1, 1, 0);
    g_object_set_data(G_OBJECT(w), "e1", (gpointer)e1);
    g_object_set_data(G_OBJECT(w), "e2", (gpointer)e2);
    g_object_set_data(G_OBJECT(w), "buffer", (gpointer)buffer);
    g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(clicked), pw);
    g_signal_connect_swapped(G_OBJECT(w), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);
    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
    gtk_window_set_default(gtkwin, w);

    w=gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_box_pack_start(hbox, w, 1, 1, 0);
    g_signal_connect_swapped(G_OBJECT(w), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);

    gtk_widget_show_all(GTK_WIDGET(gtkwin));
}
