#include "config.h"

#include <gtk/gtk.h>

#include "dock.h"
#include "globals.h"
#include "filestuff.h"
#include "die.h"
#include "popup_alert.h"
#include "popup_passphrase.h"
#include "popup_editentry.h"
#include "popup_arrangeentries.h"

void SaveEntries(GtkObject *b, gpointer data){
    if(status==0) return;
    
    if(save_passwords(passwords)){
        ErrAlert("Error saving passwords", "Error saving passwords");
    } else {
        changed=0;
        Alert(GTK_MESSAGE_INFO, "Success", "Passwords saved!");
    }
}

void NewEntry(GtkObject *b, gpointer data){
    Password pw;

    if(status==0) return;

    if((pw=password_new("-- NEW --", "- ENTRY -", ""))==NULL){
        ErrAlert("New Entry Failed", "Failed to create a new password entry");
        return;
    }
    if(pwlist_push(passwords, pw)){
        ErrAlert("New Entry Failed", "Failed to add new password to list");
        return;
    }
    changed=1;
    passwords_idx=pwlist_count(passwords)-1;
    redraw_dock();
    EditEntry(b, pw);
}

void DeleteEntry(GtkObject *b, gpointer data){
    if(status==0) return;

    if(pwlist_count(passwords)<=0){
        Alert(GTK_MESSAGE_INFO, "No passwords!", "There are no passwords to delete.");
        return;
    }
    if(pwlist_free_i(passwords, passwords_idx)){
        ErrAlert("Deletion Failed", "Failed to delete the current entry");
    }
    changed=1;
    if(passwords_idx>=pwlist_count(passwords)) passwords_idx=0;
    redraw_dock();
}

#define DAMNIT() { if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin)); debug(DEBUG_ERR, "Couldn't create GTK widgets"); return; }

void PopupDialog(void){
    GtkBox *vbox;
    GtkWidget *button;

    if(gtkwin!=NULL){
        gtk_window_present(gtkwin);
        return;
    }

    if(status==0) return;

    gtkwin=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
    if(gtkwin==NULL) DAMNIT();
    gtk_window_set_title(GTK_WINDOW(gtkwin), "Password Functions");
    g_signal_connect(G_OBJECT(gtkwin), "destroy", G_CALLBACK(gtk_widget_destroyed), &gtkwin);

    vbox=GTK_BOX(gtk_vbutton_box_new());
    if(vbox==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(gtkwin), GTK_WIDGET(vbox));

    button=gtk_button_new_with_label("Enter New Password");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(NewEntry), NULL);

    button=gtk_button_new_with_label("Delete Current Password");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(DeleteEntry), NULL);

    button=gtk_button_new_with_label("Edit Current Password");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(EditEntry), NULL);

    button=gtk_button_new_with_label("Arrange Passwords");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(ArrangeEntries), NULL);

    button=gtk_button_new_with_label("Save Passwords");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(SaveEntries), NULL);

    button=gtk_button_new_with_label("Change Passphrase");
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(ChangePassphrase), NULL);

    button=gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_box_pack_start(vbox, button, 1, 1, 0);
    g_signal_connect_swapped(G_OBJECT(button), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);

    gtk_widget_show_all(GTK_WIDGET(gtkwin));
}
