#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <mcrypt.h>
#include <gtk/gtk.h>

#include "wmpasman.h"
#include "globals.h"
#include "die.h"
#include "crypto.h"
#include "filestuff.h"
#include "password.h"
#include "popup_alert.h"
#include "gtk_mytextbuffer.h"

#define DAMNIT() { if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin)); debug(DEBUG_ERR, "Couldn't create GTK widgets"); return -1; }

static void toggle_visibility(GtkToggleButton *b, GtkTextBuffer *entry){
    my_text_buffer_set_visibility(entry, gtk_toggle_button_get_active(b));
}

static void clicked(GObject *b, gpointer data){
    void (*callback)(char *algo, char *text, gpointer data);
    GtkTextBuffer *entry;
    GtkOptionMenu *menu;

    callback=(void (*)(char *, char *, gpointer))g_object_get_data(b, "callback");
    entry=GTK_TEXT_BUFFER(g_object_get_data(b, "entry"));
    menu=GTK_OPTION_MENU(g_object_get_data(b, "menu"));
    (*callback)((char *)gtk_label_get_text(GTK_LABEL(GTK_BIN(menu)->child)), my_text_buffer_get_buffer(entry), data);
}

static void menu_changed(GtkOptionMenu *m, GtkWidget *b){
    gtk_widget_set_sensitive(b, (gtk_option_menu_get_history(m)!=0));
}

static char **cyphers=NULL;
static int num_cyphers=-1;

static int compr(const void *a, const void *b){
    return strcasecmp(*(char **)a, *(char **)b);
}

static int make_passphrase_win(const char *txt, void (*callback)(char *algo, char *text, gpointer data), gpointer data){
    GtkBox *vbox, *hbox;
    GtkWidget *w, *w2, *opt, *m;
    GtkScrolledWindow *scrolled;
    GtkTextBuffer *entry;
    int i;

    if(gtkwin!=NULL) gtk_widget_destroy(GTK_WIDGET(gtkwin));
    gtkwin=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
    if(gtkwin==NULL) DAMNIT();
    gtk_container_set_border_width(GTK_CONTAINER(gtkwin), 5);
    g_signal_connect(G_OBJECT(gtkwin), "destroy", G_CALLBACK(gtk_widget_destroyed), &gtkwin);

    vbox=GTK_BOX(gtk_vbox_new(0,2));
    if(vbox==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(gtkwin), GTK_WIDGET(vbox));

    hbox=GTK_BOX(gtk_hbox_new(0,2));
    w=gtk_alignment_new(0.0, 0.5, 0, 0);
    if(hbox==NULL || w==NULL) DAMNIT();
    gtk_container_add(GTK_CONTAINER(w), GTK_WIDGET(hbox));
    gtk_box_pack_start(vbox, w, 0, 0, 0);

    w=gtk_label_new(txt);
    gtk_box_pack_start(hbox, w, 0, 0, 0);

    if((w2=gtk_check_button_new())==NULL) DAMNIT();
    gtk_box_pack_start(hbox, w2, 0, 0, 0);

    entry=my_text_buffer_new(password_edit_buffer, MAX_PASSWORD_LEN);
    if(entry==NULL) DAMNIT();
    w=gtk_text_view_new_with_buffer(entry);

    scrolled=GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
    gtk_scrolled_window_set_policy(scrolled, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(scrolled, GTK_SHADOW_IN);
    gtk_container_add(GTK_CONTAINER(scrolled), w);
    gtk_box_pack_start(vbox, GTK_WIDGET(scrolled), 1, 1, 5);
    gtk_widget_grab_focus(w);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w2), my_text_buffer_get_visibility(entry));
    g_signal_connect(G_OBJECT(w2), "toggled", G_CALLBACK(toggle_visibility), entry);

    m=gtk_menu_new();
    w=gtk_menu_item_new_with_label("Select a cypher");
    gtk_menu_shell_append(GTK_MENU_SHELL(m), w);
    if(num_cyphers<0){
        cyphers=mcrypt_list_algorithms(NULL, &num_cyphers);
        if(num_cyphers<0 || cyphers==NULL){
            gtk_widget_destroy(GTK_WIDGET(gtkwin));
            debug(DEBUG_ERR, "Couldn't retrieve any cyphers");
            return -1;
        }
        qsort(cyphers, num_cyphers, sizeof(char *), compr);
    }
    for(i=0; i<num_cyphers; i++){
        w=gtk_menu_item_new_with_label(cyphers[i]);
        gtk_menu_shell_append(GTK_MENU_SHELL(m), w);
    }
    opt=gtk_option_menu_new();
    gtk_option_menu_set_menu(GTK_OPTION_MENU(opt), m);
    gtk_option_menu_set_history(GTK_OPTION_MENU(opt), 0);
    gtk_box_pack_start(vbox, GTK_WIDGET(opt), 0, 0, 0);

    hbox=GTK_BOX(gtk_hbutton_box_new());
    if(hbox==NULL) DAMNIT();
    gtk_box_pack_start(vbox, GTK_WIDGET(hbox), 0, 0, 0);

    w=gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_box_pack_start(hbox, w, 1, 1, 0);
    g_object_set_data(G_OBJECT(w), "callback", (gpointer)callback);
    g_object_set_data(G_OBJECT(w), "entry", (gpointer)entry);
    g_object_set_data(G_OBJECT(w), "menu", (gpointer)opt);
    gtk_widget_set_sensitive(w, FALSE);
    g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(clicked), data);
    g_signal_connect_swapped(G_OBJECT(w), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);
    g_signal_connect(G_OBJECT(opt), "changed", G_CALLBACK(menu_changed), w);
    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
    gtk_window_set_default(gtkwin, w);

    w=gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_box_pack_start(hbox, w, 1, 1, 0);
    g_signal_connect_swapped(G_OBJECT(w), "clicked", G_CALLBACK(gtk_widget_destroy), gtkwin);

    return 0;
}

static void GetPassphrase_callback(char *algo, char *text, gpointer data){
    if(status!=0) return;
    
    if(crypto_set_passphrase(algo, text)){
        ErrAlert("Couldn't set passphrase", "Error setting new passphrase");
        return;
    }
    passwords=load_passwords();
    changed=0;
    if(passwords==NULL){
        ErrAlert("Couldn't load passwords", "Error loading passwords");
        return;
    }
    status=1;
}

void GetPassphrase(void){
    if(status!=0) return;
    if(make_passphrase_win("Enter Passphrase", GetPassphrase_callback, NULL)) return;
    gtk_window_set_title(GTK_WINDOW(gtkwin), "Get Passphrase");

    gtk_widget_show_all(GTK_WIDGET(gtkwin));
}

static void ChangePassphrase_callback(char *algo, char *text, gpointer data){
    if(status==0) return;
    
    if(crypto_set_passphrase(algo, text)){
        ErrAlert("Couldn't set passphrase", "Error setting new passphrase");
        return;
    }
    if(pwlist_recrypt_passes(passwords)){
        ErrAlert("Couldn't change passphrase", "Couldn't change passphrase");
        pwlist_free_all(passwords);
        crypto_forget_passphrase();
        status=0;
        changed=0;
        return;
    }
    changed=1;
}

void ChangePassphrase(GtkObject *b, gpointer data){
    if(status==0) return;
    if(make_passphrase_win("Enter New Passphrase", ChangePassphrase_callback, NULL)) return;
    gtk_window_set_title(GTK_WINDOW(gtkwin), "Change Passphrase");

    gtk_widget_show_all(GTK_WIDGET(gtkwin));
}
