#include "../config.h"

#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/resource.h>

#include "wmpasman.h"
#include "globals.h"
#include "secure.h"
#include "die.h"

/* unistd.h is supposed to declare these, but it doesn't */
int setresuid(uid_t ruid, uid_t euid, uid_t suid);
int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);
int setresgid(gid_t rgid, gid_t egid, gid_t sgid);
int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);

/* Adapted from a paper at
 * http://www.cs.berkeley.edu/~daw/papers/setuid-usenix02.pdf
 */ 
void drop_perms(void){
    uid_t ruid, euid, suid;
    uid_t new_uid;
    gid_t rgid, egid, sgid;
    gid_t new_gid;

    errno=0;
    new_uid=getuid();
    new_gid=getgid();
    debug(DEBUG_BLEH, "dropping permissions to UID %d GID %d", new_uid, new_gid);
    if(new_uid==0) die("cannot run as root. Sorry.");
    if(new_gid==0) die("cannot run as group 0. Sorry.");

    if(setresgid(new_gid, new_gid, new_gid)!=0)
        die("failed to drop group permissions");
    if(getresgid(&rgid, &egid, &sgid)!=0)
        die("failed to drop group permissions");
    if(rgid!=new_gid || egid!=new_gid || sgid!=new_gid)
        die("failed to drop group permissions");

    if(setresuid(new_uid, new_uid, new_uid)!=0)
        die("failed to drop user permissions");
    if(getresuid(&ruid, &euid, &suid)!=0)
        die("failed to drop user permissions");
    if(ruid!=new_uid || euid!=new_uid || suid!=new_uid)
        die("failed to drop user permissions");
}

#define SIZE MAX_PHRASEHASH_LEN*2+MAX_PASSWORD_LEN*2
void secure_startup(void){
    struct rlimit r;
    long pagesize;
    void *v;

    debug(DEBUG_BLEH, "Allocating %d bytes of secure memory...", SIZE);
    pagesize=sysconf(_SC_PAGESIZE);
    if(pagesize<1) die("WTF? sysconf(_SC_PAGESIZE) returned %ld", pagesize);
    debug(DEBUG_BLEH, "Page size: %ld", pagesize);
    if((errno=posix_memalign(&v, pagesize, SIZE))) die("Error in aligned memory allocation");
    if(mlock(v, SIZE)){
        if(errno!=EPERM) die("Memory locking error");
        errno=0;
        debug(DEBUG_WARN, "WARNING: Using insecure memory!");
    } else {
        errno=0;
        debug(DEBUG_INFO, "Allocated %d bytes secure memory (%ld pages)", SIZE, (SIZE+pagesize-1)/pagesize);
    }

    passphrase_buffer=(char *)v;
    passphrase_buffer2=((char *)v)+MAX_PHRASEHASH_LEN;
    password_buffer=((char *)v)+MAX_PHRASEHASH_LEN*2;
    password_edit_buffer=((char *)v)+MAX_PHRASEHASH_LEN*2+MAX_PASSWORD_LEN;
    
    memset(password_buffer, 0, MAX_PASSWORD_LEN);
    memset(password_edit_buffer, 0, MAX_PASSWORD_LEN);
    memset(passphrase_buffer, 0, MAX_PHRASEHASH_LEN);
    memset(passphrase_buffer2, 0, MAX_PHRASEHASH_LEN);

    debug(DEBUG_BLEH, "disabling core files...");
    r.rlim_cur=r.rlim_max=0;
    if(setrlimit(RLIMIT_CORE, &r)){
        debug(DEBUG_WARN, "WARNING: Couldn't disable corefiles");
    }

    drop_perms();
}
